﻿using System.IO;
using System.Net;
using System.Text;

namespace TestProject
{
    /// <summary>
    /// HTTP 헬퍼
    /// </summary>
    public static class HTTPHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region POST 요청 스트림 구하기 - GetStreamPOSTRequest(url, contentType, encoding, argument)

        /// <summary>
        /// POST 요청 스트림 구하기
        /// </summary>
        /// <param name="url">URL</param>
        /// <param name="contentType">컨텐츠 타입</param>
        /// <param name="encoding">인코딩</param>
        /// <param name="argument">인자 문자열</param>
        /// <returns>POST 요청 스트림</returns>
        public static Stream GetStreamPOSTRequest(string url, string contentType, Encoding encoding, string argument)
        {
            byte[] contentArray = encoding.GetBytes(argument);
            
            HttpWebRequest request = WebRequest.Create(url) as HttpWebRequest;

            request.ProtocolVersion           = HttpVersion.Version11;
            request.AllowAutoRedirect         = true;
            request.AllowWriteStreamBuffering = true;
            request.Method                    = WebRequestMethods.Http.Post;
            request.ContentType               = contentType;
            request.ContentLength             = contentArray.Length;

            using(Stream requestStream = request.GetRequestStream())
            {
                requestStream.Write(contentArray, 0, contentArray.Length);
            }

            HttpWebResponse response = request.GetResponse() as HttpWebResponse;

            Stream responseStream = response.GetResponseStream();

            return responseStream;
        }

        #endregion
        #region POST 요청 문자열 구하기 - GetStringPOSTRequest(url, contentType, encoding, argument)

        /// <summary>
        /// POST 요청 문자열 구하기
        /// </summary>
        /// <param name="url">URL</param>
        /// <param name="contentType">컨텐츠 타입</param>
        /// <param name="encoding">인코딩</param>
        /// <param name="argument">인자 문자열</param>
        /// <returns>POST 요청 문자열</returns>
        public static string GetStringPOSTRequest(string url, string contentType, Encoding encoding, string argument)
        {
            using(Stream stream = GetStreamPOSTRequest(url, contentType, encoding, argument))
            {
                using(StreamReader reader = new StreamReader(stream))
                {
                    string result = reader.ReadToEnd();

                    return result;
                }
            }
        }

        #endregion
    }
}