﻿using System.Collections.Generic;
using System.Text;

using Newtonsoft.Json;

namespace TestProject
{
    /// <summary>
    /// NATS 헬퍼
    /// </summary>
    public static class NATSHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 서버 정보 구하기 - GetVARZ(serverAddress, serverPort)

        /// <summary>
        /// 서버 정보 구하기
        /// </summary>
        /// <param name="serverAddress">서버 주소</param>
        /// <param name="serverPort">서버 포트</param>
        /// <returns>서버 정보</returns>
        public static varz GetVARZ(string serverAddress, int serverPort)
        {
            string   url         = $"http://{serverAddress}:{serverPort}/varz";
            string   contentType = "application/json; charset=utf8";
            Encoding encoding    = Encoding.UTF8;
            string   argument    = string.Empty;

            string json = HTTPHelper.GetStringPOSTRequest(url, contentType, encoding, argument);

            varz varz = JsonConvert.DeserializeObject<varz>(json);

            return varz;
        }

        #endregion
        #region 연결 정보 구하기 - GetCONNZ(serverAddress, serverPort, limit, offset, includeSubscriptionList)

        /// <summary>
        /// 연결 정보 구하기
        /// </summary>
        /// <param name="serverAddress">서버 주소</param>
        /// <param name="serverPort">서버 포트</param>
        /// <param name="itemLimitCount">항목 제한 수</param>
        /// <param name="itemOffset">항목 오프셋</param>
        /// <param name="includeSubscriptionList">구독 리스트 포함 여부</param>
        /// <returns>연결 정보</returns>
        public static connz GetCONNZ(string serverAddress, int serverPort, int itemLimitCount, int itemOffset, bool includeSubscriptionList = true)
        {
            string   url         = $"http://{serverAddress}:{serverPort}/connz?limit={itemLimitCount}&offset={itemOffset}" + (includeSubscriptionList ? "&subs=1" : string.Empty);
            string   contentType = "application/json; charset=utf8";
            Encoding encoding    = Encoding.UTF8;
            string   argument    = string.Empty;

            string json = HTTPHelper.GetStringPOSTRequest(url, contentType, encoding, argument);

            connz connz = JsonConvert.DeserializeObject<connz>(json);

            return connz;
        }

        #endregion
        #region 연결 정보 구하기 - GetCONNZ(serverAddress, serverPort)

        /// <summary>
        /// 연결 정보 구하기
        /// </summary>
        /// <param name="serverAddress">서버 주소</param>
        /// <param name="serverPort">서버 포트</param>
        /// <returns>연결 정보</returns>
        public static connz GetCONNZ(string serverAddress, int serverPort)
        {
            int limit   = 1024;
            int current = 0;

            connz resultCONNZ = new connz();

            resultCONNZ.connections = new List<connection>();
            
            while(true)
            {
                connz connz = NATSHelper.GetCONNZ(serverAddress, serverPort, limit, current, true);

                resultCONNZ.server_id       =  connz.server_id;
                resultCONNZ.now             =  connz.now;
                resultCONNZ.num_connections += connz.num_connections;
                resultCONNZ.total           =  connz.total;
                resultCONNZ.offset          =  connz.offset;
                resultCONNZ.limit           =  connz.limit;

                foreach(connection connection in connz.connections)
                {
                    resultCONNZ.connections.Add(connection);
                }

                current += limit;

                if(current >= connz.total)
                {
                    break;
                }
            }

            return resultCONNZ;
        }

        #endregion
        #region 라우팅 정보 구하기 - GetROUTEZ(serverAddress, serverPort, includeSubscriptionList)

        /// <summary>
        /// 라우팅 정보 구하기
        /// </summary>
        /// <param name="serverAddress">서버 주소</param>
        /// <param name="serverPort">서버 포트</param>
        /// <param name="includeSubscriptionList">구독 리스트 포함 여부</param>
        /// <returns>라우팅 정보</returns>
        public static routez GetROUTEZ(string serverAddress, int serverPort, bool includeSubscriptionList = true)
        {
            string   url         = $"http://{serverAddress}:{serverPort}/routez" + (includeSubscriptionList ? "?subs=1" : string.Empty);
            string   contentType = "application/json; charset=utf8";
            Encoding encoding    = Encoding.UTF8;
            string   argument    = string.Empty;

            string json = HTTPHelper.GetStringPOSTRequest(url, contentType, encoding, argument);

            routez routez = JsonConvert.DeserializeObject<routez>(json);

            return routez;
        }

        #endregion
        #region 구독 정보 구하기 - GetSUBSZ(serverAddress, serverPort, limit, offset, includeSubscriptionList)

        /// <summary>
        /// 구독 정보 구하기
        /// </summary>
        /// <param name="serverAddress">서버 주소</param>
        /// <param name="serverPort">서버 포트</param>
        /// <param name="itemLimitCount">항목 제한 수</param>
        /// <param name="itemOffset">항목 오프셋</param>
        /// <param name="includeSubscriptionList">구독 리스트 포함 여부</param>
        /// <returns>구독 정보</returns>
        public static subsz GetSUBSZ(string serverAddress, int serverPort, int itemLimitCount, int itemOffset, bool includeSubscriptionList = true)
        {
            string   url         = $"http://{serverAddress}:{serverPort}/subsz?limit={itemLimitCount}&offset={itemOffset}" + (includeSubscriptionList ? "&subs=1" : string.Empty);
            string   contentType = "application/json; charset=utf8";
            Encoding encoding    = Encoding.UTF8;
            string   argument    = string.Empty;

            string json = HTTPHelper.GetStringPOSTRequest(url, contentType, encoding, argument);

            subsz subsz = JsonConvert.DeserializeObject<subsz>(json);

            return subsz;
        }

        #endregion
        #region 구독 정보 구하기 - GetSUBSZ(serverAddress, serverPort)

        /// <summary>
        /// 구독 정보 구하기
        /// </summary>
        /// <param name="serverAddress">서버 주소</param>
        /// <param name="serverPort">서버 포트</param>
        /// <returns>구독 정보</returns>
        public static subsz GetSUBSZ(string serverAddress, int serverPort)
        {
            int limit   = 1024;
            int current = 0;
            
            subsz resultSUBSZ = new subsz();

            resultSUBSZ.subscriptions_list = new List<subscription>();

            while(true)
            {
                subsz subsz = NATSHelper.GetSUBSZ("127.0.0.1", 20030, limit, current);
            
                resultSUBSZ.num_subscriptions = subsz.num_subscriptions;
                resultSUBSZ.num_cache         =  subsz.num_cache;
                resultSUBSZ.num_inserts       =  subsz.num_inserts;
                resultSUBSZ.num_removes       =  subsz.num_removes;
                resultSUBSZ.num_matches       =  subsz.num_matches;
                resultSUBSZ.cache_hit_rate    =  subsz.cache_hit_rate;
                resultSUBSZ.max_fanout        =  subsz.max_fanout;
                resultSUBSZ.avg_fanout        =  subsz.avg_fanout;
                resultSUBSZ.total             += subsz.total;
                resultSUBSZ.offset            =  subsz.offset;
                resultSUBSZ.limit             =  subsz.limit;

                foreach(subscription subscription in subsz.subscriptions_list)
                {
                    resultSUBSZ.subscriptions_list.Add(subscription);
                }
            
                if(subsz.total < limit)
                {
                    break;
                }
            
                current += limit;
            }

            return resultSUBSZ;
        }

        #endregion
    }
}